@@ -1,13 +1,17 @@
 isCol <- function(data, varname) {
   !is.null(data) & varname %in% names(data)
 }
-is.log.effect <- function(x)
+
+is_log_effect <- function(x)
   x %in% c("PLN", "IRLN", "MLN")
-is.relative.effect <- function(x)
+
+is_relative_effect <- function(x)
   x %in% c("HR", "OR", "RR", "IRR", "ROM", "DOR")
-is.untransformed <- function(x)
+
+is_untransformed <- function(x)
   x %in% c("COR", "PRAW", "IR", "RD", "IRD", "MD", "SMD", "MRAW")
-is.wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
+
+is_wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
   if (is.numeric(x))
     res <- abs(x - round(x)) < tol
   else
@@ -15,13 +19,18 @@ is.wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
   ##
   res
 }
-is.zero <- function(x, n = 10)
+
+is_zero <- function(x, n = 10)
   abs(x) < n * .Machine$double.eps
-is.cor <- function(x)
+
+is_cor <- function(x)
   x %in% gs("sm4cor")
-is.mean <- function(x)
+
+is_mean <- function(x)
   x %in% gs("sm4mean")
-is.prop <- function(x)
+
+is_prop <- function(x)
   x %in% gs("sm4prop")
-is.rate <- function(x)
+
+is_rate <- function(x)
   x %in% gs("sm4rate")
