@@ -24,6 +24,7 @@
 #' @param cluster An optional vector specifying which estimates come
 #'   from the same cluster resulting in the use of a three-level
 #'   meta-analysis model.
+#' @param rho Assumed correlation of estimates within a cluster.
 #' @param method A character string indicating which method is to be
 #'   used for pooling of studies. One of \code{"MH"},
 #'   \code{"Inverse"}, \code{"Cochran"}, or \code{"GLMM"} can be
@@ -193,20 +194,22 @@
 #' \subsection{Meta-analysis method}{
 #' 
 #' By default, both common effect and random effects models are
-#' considered (see arguments \code{common} and
-#' \code{random}). If \code{method} is \code{"MH"} (default), the
-#' Mantel-Haenszel method is used to calculate the common effect
-#' estimate (Greenland & Robbins, 1985); if \code{method} is
-#' \code{"Inverse"}, inverse variance weighting is used for pooling;
-#' if \code{method} is \code{"Cochran"}, the Cochran method is used
-#' for pooling (Bayne-Jones, 1964, Chapter 8).
+#' considered (see arguments \code{common} and \code{random}). If
+#' \code{method} is \code{"MH"} (default), the Mantel-Haenszel method
+#' is used to calculate the common effect estimate (Greenland &
+#' Robbins, 1985); if \code{method} is \code{"Inverse"}, inverse
+#' variance weighting is used for pooling; if \code{method} is
+#' \code{"Cochran"}, the Cochran method is used for pooling
+#' (Bayne-Jones, 1964, Chapter 8). For these three methods, the random
+#' effects estimate is always based on the inverse variance method.
 #' 
 #' A distinctive and frequently overlooked advantage of incidence
 #' rates is that individual patient data (IPD) can be extracted from
 #' count data. Accordingly, statistical methods for IPD, i.e.,
 #' generalised linear mixed models, can be utilised in a meta-analysis
 #' of incidence rate ratios (Stijnen et al., 2010). These methods are
-#' available (argument \code{method = "GLMM"}) by calling the
+#' available (argument \code{method = "GLMM"}) for the common effect
+#' and random effects model by calling the
 #' \code{\link[metafor]{rma.glmm}} function from R package
 #' \bold{metafor} internally.
 #'
@@ -391,7 +394,8 @@
 
 metainc <- function(event.e, time.e, event.c, time.c, studlab,
                     ##
-                    data = NULL, subset = NULL, exclude = NULL, cluster = NULL,
+                    data = NULL, subset = NULL, exclude = NULL,
+                    cluster = NULL, rho = 0,
                     ##
                     method = if (sm == "IRSD") "Inverse" else "MH",
                     sm = gs("sminc"),
@@ -402,7 +406,11 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
                     common = gs("common"),
                     random = gs("random") | !is.null(tau.preset),
                     overall = common | random,
-                    overall.hetstat = common | random,
+                    overall.hetstat =
+                      if (is.null(gs("overall.hetstat")))
+                        common | random
+                      else
+                        gs("overall.hetstat"),   
                     prediction = gs("prediction") | !missing(method.predict),
                     ##
                     method.tau =
@@ -463,6 +471,8 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   ## (1) Check arguments
   ##
   ##
+  chknumeric(rho, min = -1, max = 1)
+  ##
   chknull(sm)
   sm <- setchar(sm, gs("sm4inc"))
   ##
@@ -926,7 +936,6 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   ##
   if (three.level) {
     chkmlm(method.tau, missing.method.tau, method.predict,
-           by, tau.common, missing.tau.common,
            method, missing.method)
     ##
     common <- FALSE
@@ -1066,7 +1075,7 @@ metainc <- function(event.e, time.e, event.c, time.c, studlab,
   ##
   m <- metagen(TE, seTE, studlab,
                exclude = if (missing.exclude) NULL else exclude,
-               cluster = cluster,
+               cluster = cluster, rho = rho,
                ##
                sm = sm,
                level = level,
