@@ -566,7 +566,7 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
               upper = c(upper.i, NA, upper.TE.s),
               ##
               three.level = x$three.level,
-              cluster = x$cluster,
+              cluster = x$cluster, rho = x$rho,
               ##
               k = x$k, k.study = x$k.study, k.all = x$k.all, k.TE = x$k.TE,
               ##
@@ -621,6 +621,9 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
               pscale = x$pscale,
               irscale = x$irscale, irunit = x$irunit,
               ##
+              label.e = x$label.e,
+              label.c = x$label.c,
+              ##
               text.common = x$text.common, text.random = x$text.random,
               text.predict = x$text.predict,
               text.w.common = x$text.w.common, text.w.random = x$text.w.random,
