@@ -21,6 +21,20 @@
 #'   effects model should be considered for second meta-analysis.
 #' @param prediction2 A logical indicating whether prediction interval
 #'   should be considered for second meta-analysis.
+#' @param label1 Default setting for arguments 'label1.common',
+#'   'label1.random', 'label1.predict' and 'label1.subgroup'.
+#' @param label2 Default setting for arguments 'label2.common',
+#'   'label2.random', 'label2.predict' and 'label2.subgroup'.
+#' @param label1.common A character string ...
+#' @param label2.common A character string ...
+#' @param label1.random A character string ... (default label for
+#'   arguments 'hetlabel1' and 'taulabel1').
+#' @param label2.random A character string ... (default label for
+#'   arguments 'hetlabel2' and 'taulabel2').
+#' @param label1.predict A character string ...
+#' @param label2.predict A character string ...
+#' @param label1.subgroup A character string ...
+#' @param label2.subgroup A character string ...
 #' @param text.pooled1 A character string used in printouts and forest
 #'   plot to label the results from the first meta-analysis.
 #' @param text.pooled2 A character string used in printouts and forest
@@ -31,10 +45,6 @@
 #' @param text.w.pooled2 A character string used to label weights of
 #'   the second meta-analysis; can be of same length as the number of
 #'   pooled estimates requested in argument \code{pooled1}.
-#' @param label1 Default setting for arguments 'hetlabel1' and
-#'   'taulabel1'.
-#' @param label2 Default setting for arguments 'hetlabel2' and
-#'   'taulabel2'.
 #' @param hetlabel1 A character string used to label heterogeneity
 #'   statistics of the first meta-analysis.
 #' @param hetlabel2 A character string used to label heterogeneity
@@ -212,77 +222,89 @@
 #'   \code{\link{metaadd}}
 #' 
 #' @examples
-#' # Print results with more significant digits
+#' # Print results with more significant digits and do not show confidence
+#' # intervals for tau^2 and tau
 #' oldset <- settings.meta(digits = 6, digits.stat = 4, digits.pval = 6,
-#'                         digits.Q = 6, digits.I2 = 4, digits.H = 4)
+#'   digits.Q = 6, digits.I2 = 4, digits.H = 4,
+#'   print.tau2.ci = FALSE, print.tau.ci = FALSE)
 #' oldopts <- options(width = 120)
 #' 
 #' data(Fleiss1993bin)
 #' 
 #' # Mantel-Haenszel method
 #' m1 <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
-#'   studlab = paste(study, year), sm = "OR",
-#'   text.common = "Mantel-Haenszel method", text.w.common = "MH",
-#'   text.random = "Random effects model (IV)", text.w.random = "RE-IV")
-#' 
+#'   studlab = paste(study, year), sm = "OR")
 #' # Peto method
-#' m2 <- update(m1, method = "Peto",
-#'   text.common = "Peto method", text.w.common = "Peto",
-#'   text.random = "Random effects model (Peto)", text.w.random = "RE-Peto")
-#' 
-#' # Inverse variance method
-#' m3 <- update(m2, method = "Inverse", random = FALSE,
-#'   text.common = "Inverse-variance method", text.w.common = "IV")
+#' m2 <- update(m1, method = "Peto")
+#' # Inverse variance method (only common effect model)
+#' m3 <- update(m2, method = "Inverse", random = FALSE)
 #' 
 #' # Merge results from MH and Peto method
-#' # - show individual results for MH method (as this is the first meta-analysis)
+#' # - show individual results for MH method
+#' #   (as this is the first meta-analysis)
 #' # - keep all additional information from Peto meta-analysis (i.e.,
 #' #   weights, Q statistic and I2 statistic)
 #' m12 <- metamerge(m1, m2,
-#'   taulabel1 = "REML-IV", taulabel2 = "REML-Peto",
+#'   label1 = "REML", label2 = "REML-Peto",
+#'   label1.common = "MH", label2.common = "Peto", 
+#'   text.common1 = "Mantel-Haenszel method",
+#'   text.common2 = "Peto method",
+#'   text.w.random1 = "REML", text.w.random2 = "REML-Peto",
 #'   hetlabel1 = "MH/IV", hetlabel2 = "Peto",
 #'   keep = TRUE)
-#' m12
 #' 
-#' # Add results from inverse variance method
+#' # Add common effect results from inverse variance method
 #' # - keep weights from IV meta-analysis
 #' # - Q and I2 statistic are identical for sm = "MH" and sm = "Inverse"
 #' #   as inverse variance method is used for sm = "MH" under random
 #' #   effects model
-#' m123 <- metamerge(m12, m3, keep.w = TRUE)
+#' m123 <- metamerge(m12, m3,
+#'   label2 = "IV",
+#'   text.common2 = "Inverse variance method",
+#'   keep.w = TRUE)
 #' summary(m123)
 #' \dontrun{
 #' forest(m123, digits = 6)
 #' 
 #' # Merge results (show individual results for Peto method)
 #' m21 <- metamerge(m2, m1,
-#'   taulabel1 = "REML-Peto", taulabel2 = "REML-IV",
+#'   label1 = "REML-Peto", label2 = "REML",
+#'   label1.common = "Peto", label2.common = "MH", 
 #'   hetlabel1 = "Peto", hetlabel2 = "MH/IV",
+#'   text.common1 = "Peto method",
+#'   text.common2 = "Mantel-Haenszel method",
 #'   keep = TRUE)
-#' m213 <- metamerge(m21, m3, keep.w = TRUE)
+#' 
+#' # Add results from inverse variance method
+#' # - keep weights from IV meta-analysis
+#' # - Q and I2 statistic are identical for sm = "MH" and sm = "Inverse"
+#' #   as inverse variance method is used for sm = "MH" under random
+#' #   effects model
+#' m213 <- metamerge(m21, m3,
+#'   label2 = "IV",
+#'   text.common2 = "Inverse variance method",
+#'   keep.w = TRUE)
 #' summary(m213)
 #' 
 #' # Random effects method using ML estimator for between-study variance tau2
-#' m4 <- update(m1, common = FALSE, method.tau = "ML",
-#'   text.random = "Random effects model (ML)", text.w.random = "RE-ML")
+#' m4 <- update(m1, common = FALSE, method.tau = "ML")
 #' 
 #' # Use DerSimonian-Laird estimator for tau2
-#' m5 <- update(m4, method.tau = "DL",
-#'   text.random = "Random effects model (DL)", text.w.random = "RE-DL")
+#' m5 <- update(m4, method.tau = "DL")
 #' 
 #' # Use Paule-Mandel estimator for tau2
-#' m6 <- update(m4, method.tau = "PM",
-#'   text.random = "Random effects model (PM)", text.w.random = "RE-PM")
+#' m6 <- update(m4, method.tau = "PM")
 #' 
 #' # Merge random effects results for ML and DL estimators
 #' # - keep weights for DL estimator (which are different from ML)
-#' m45 <- metamerge(m4, m5, taulabel1 = "ML", taulabel2 = "DL",
-#'    keep.w = TRUE)
+#' m45 <- metamerge(m4, m5, label1 = "ML", label2 = "DL",
+#'   text.w.random1 = "RE-ML", text.w.random2 = "RE-DL", keep.w = TRUE)
 #' summary(m45)
 #' 
 #' # Add results for PM estimator
 #' # - keep weights
-#' m456 <- metamerge(m45, m6, taulabel2 = "PM", keep.w = TRUE)
+#' m456 <- metamerge(m45, m6, label2 = "PM",
+#'   text.w.random2 = "RE-PM", keep.w = TRUE)
 #' summary(m456)
 #' 
 #' m123456 <- metamerge(m123, m456)
@@ -291,35 +313,32 @@
 #' # Use Hartung-Knapp confidence intervals
 #' # - do not keep information on Q, I2 and weights
 #' m7 <- update(m4, method.random.ci = "HK",
-#'   text.random = "Hartung-Knapp method (REML)")
+#'   text.random = "Hartung-Knapp method")
 #' m8 <- update(m5, method.random.ci = "HK",
-#'   text.random = "Hartung-Knapp method (DL)")
+#'   text.random = "Hartung-Knapp method")
 #' m9 <- update(m6, method.random.ci = "HK",
-#'   text.random = "Hartung-Knapp method (PM)")
+#'   text.random = "Hartung-Knapp method")
 #' 
 #' # Merge results for Hartung-Knapp method (with REML and DL estimator)
 #' # - RE weights for REML estimator are shown
-#' m78 <- metamerge(m7, m8)
+#' m78 <- metamerge(m7, m8, label1 = "ML", label2 = "DL")
 #' summary(m78)
 #' 
-#' m789 <- metamerge(m78, m9)
+#' m789 <- metamerge(m78, m9, label2 = "PM")
 #' summary(m789)
 #' 
 #' # Merge everything
 #' m1to9 <- metamerge(metamerge(m123, m456, keep.w = TRUE), m789)
 #' summary(m1to9)
 #' 
-#' m10 <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
-#'   studlab = paste(study, year), sm = "OR", method = "GLMM",
-#'   text.common = "Common effect model (GLMM)",
-#'   text.random = "Random effects model (GLMM)")
+#' m10 <- update(m1, method = "GLMM")
 #' 
 #' m.all <- metamerge(m1to9, m10, keep.Q = TRUE,
-#'   taulabel2 = "ML-GLMM", hetlabel2 = "GLMM")
+#'   label2 = "GLMM", taulabel2 = "ML-GLMM")
 #' summary(m.all)
 #' 
 #' forest(m.all, layout = "JAMA")
-#' forest(m.all)
+#' forest(m.all, details = TRUE)
 #' }
 #' 
 #' settings.meta(oldset)
@@ -337,12 +356,19 @@ metamerge <- function(meta1, meta2,
                       random2 = meta2$random,
                       prediction2 = meta2$prediction,
                       ##
-                      text.pooled1 = "", text.pooled2 = "",
-                      text.w.pooled1 = "", text.w.pooled2 = "",
+                      label1 = NULL, label2 = NULL,
+                      label1.common = label1, label2.common = label2,
+                      label1.random = label1, label2.random = label2,
+                      label1.predict = label1, label2.predict = label2,
+                      label1.subgroup = label1, label2.subgroup = label2,
                       ##
-                      label1 = "", label2 = "",
-                      hetlabel1 = label1, hetlabel2 = label2,
-                      taulabel1 = label1, taulabel2 = label2,
+                      hetlabel1 = label1.random,
+                      hetlabel2 = label2.random,
+                      taulabel1 = label1.random,
+                      taulabel2 = label2.random,
+                      ##
+                      text.pooled1 = NULL, text.pooled2 = NULL,
+                      text.w.pooled1 = NULL, text.w.pooled2 = NULL,
                       ##
                       text.common1 = text.pooled1,
                       text.common2 = text.pooled2,
@@ -380,18 +406,37 @@ metamerge <- function(meta1, meta2,
   if (missing(meta1))
     stop("Argument 'meta1' must be provided.",
          call. = FALSE)
-  if (missing(meta2)) {
-    warning("Argument 'meta2' not provided.",
-            call. = FALSE)
-    return(meta1)
-  }
   ##
-  if (inherits(meta1, "copas") |
-      inherits(meta1, "limitmeta") |
-      inherits(meta1, "robu"))
-    stop("Argument 'meta1' cannot be of class '",
-         class(meta1), "' (use argument 'meta2').",
-         call. = FALSE)
+  if (missing(meta2)) {
+    if (!inherits(meta1, "metamerge") & inherits(meta1, "copas"))
+      return(metamerge(update(meta1$x), meta1,
+                       label2 = if (is.null(label2)) "copas" else label2))
+    ##
+    else if (!inherits(meta1, "metamerge") & inherits(meta1, "limitmeta"))
+      return(metamerge(update(meta1$x), meta1,
+                       label2 = if (is.null(label2)) "limit" else label2))
+    ##
+    else if (!inherits(meta1, "metamerge") & inherits(meta1, "robu"))
+      stop("Argument 'meta1' cannot be of class 'robu'.",
+           call. = FALSE)
+    ##
+    else
+      return(meta1)
+  }
+  else if (!inherits(meta1, "metamerge")) {
+    if (inherits(meta1, "copas"))
+      stop("Argument 'meta1' cannot be of class 'copas' ",
+           "(use argument 'meta2').",
+           call. = FALSE)
+    else if (inherits(meta1, "limitmeta"))
+      stop("Argument 'meta1' cannot be of class 'limitmeta' ",
+           "(use argument 'meta2').",
+           call. = FALSE)
+    else if (inherits(meta1, "robu"))
+      stop("Argument 'meta1' cannot be of class 'robu' ",
+           "(use argument 'meta2').",
+           call. = FALSE)
+  }
   ##
   chkclass(meta1, "meta")
   meta1 <- updateversion(meta1)
@@ -405,15 +450,22 @@ metamerge <- function(meta1, meta2,
   if (inherits(meta2, "meta"))
     meta2 <- updateversion(meta2)
   ##
-  is.copas <- inherits(meta2, "copas")
-  is.limit <- inherits(meta2, "limitmeta")
-  is.robu <- inherits(meta2, "robu")
+  is.copas <- !inherits(meta2, "metamerge") & inherits(meta2, "copas")
+  is.limit <- !inherits(meta2, "metamerge") & inherits(meta2, "limitmeta")
+  is.robu  <- !inherits(meta2, "metamerge") & inherits(meta2, "robu")
   ##
   if (is.copas | is.limit | is.robu) {
     common2 <- FALSE
     random2 <- TRUE
     prediction2 <- FALSE
     meta2$three.level <- FALSE
+    meta2$rho <- NA
+    ##
+    if (is.copas)
+      meta2$k <- length(meta2$TE)
+    ##
+    if (is.limit)
+      meta2$tau2 <- meta2$tau^2
     ##
     keep <- FALSE
     keep.Q <- FALSE
@@ -470,38 +522,46 @@ metamerge <- function(meta1, meta2,
   chklogical(common2)
   chklogical(random2)
   ##
-  chkchar(text.pooled1, length = 1)
-  chkchar(text.pooled2, length = 1)
-  chkchar(text.common1, length = 1)
-  chkchar(text.common2, length = 1)
-  chkchar(text.random1, length = 1)
-  chkchar(text.random2, length = 1)
-  chkchar(text.predict1, length = 1)
-  chkchar(text.predict2, length = 1)
-  ##
-  chkchar(text.w.pooled1)
-  chkchar(text.w.pooled2)
-  chkchar(text.w.common1)
-  chkchar(text.w.common2)
-  chkchar(text.w.random1)
-  chkchar(text.w.random2)
-  ##
   chklogical(keep)
   chklogical(keep.Q)
   chklogical(keep.I2)
   chklogical(keep.w)
   ##
-  missing.hetlabel1 <- missing(label1) & missing(hetlabel1)
-  missing.taulabel1 <- missing(label1) & missing(taulabel1)
-  missing.hetlabel2 <- missing(label2) & missing(hetlabel2)
-  missing.taulabel2 <- missing(label2) & missing(taulabel2)
+  chkchar(label1, length = 1, NULL.ok = TRUE)
+  chkchar(label2, length = 1, NULL.ok = TRUE)
   ##
-  chkchar(label1, length = 1)
-  chkchar(label2, length = 1)
-  chkchar(hetlabel1, length = 1)
-  chkchar(taulabel1, length = 1)
-  chkchar(hetlabel2, length = 1)
-  chkchar(taulabel2, length = 1)
+  chkchar(label1.common, length = 1, NULL.ok = TRUE)
+  chkchar(label2.common, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(label1.random, length = 1, NULL.ok = TRUE)
+  chkchar(label2.random, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(label1.predict, length = 1, NULL.ok = TRUE)
+  chkchar(label2.predict, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(hetlabel1, length = 1, NULL.ok = TRUE)
+  chkchar(taulabel1, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(hetlabel2, length = 1, NULL.ok = TRUE)
+  chkchar(taulabel2, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(text.pooled1, length = 1, NULL.ok = TRUE)
+  chkchar(text.common1, length = 1, NULL.ok = TRUE)
+  chkchar(text.random1, length = 1, NULL.ok = TRUE)
+  chkchar(text.predict1, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(text.common2, length = 1, NULL.ok = TRUE)
+  chkchar(text.pooled2, length = 1, NULL.ok = TRUE)
+  chkchar(text.random2, length = 1, NULL.ok = TRUE)
+  chkchar(text.predict2, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(text.w.pooled1, length = 1, NULL.ok = TRUE)
+  chkchar(text.w.common1, length = 1, NULL.ok = TRUE)
+  chkchar(text.w.random1, length = 1, NULL.ok = TRUE)
+  ##
+  chkchar(text.w.pooled2, length = 1, NULL.ok = TRUE)
+  chkchar(text.w.common2, length = 1, NULL.ok = TRUE)
+  chkchar(text.w.random2, length = 1, NULL.ok = TRUE)
   ##
   chklogical(common)
   chklogical(random)
@@ -524,11 +584,13 @@ metamerge <- function(meta1, meta2,
   ##
   ## Check summary measures
   ##
-  samesm(meta1, meta2)
+  if (!is.robu)
+    samesm(meta1, meta2)
   ##
   ## Check original data (if available)
   ##
-  samedata(meta1, meta2)
+  if (!is.robu)
+    samedata(meta1, meta2)
   ##
   ## Check subgroup levels
   ##
@@ -556,13 +618,23 @@ metamerge <- function(meta1, meta2,
   ##     variance meta-analysis (meta2)
   ##
   
-  meta1 <- updateobj(meta1, text.common1, text.random1, text.predict1,
-                     text.w.common1, text.w.random1,
-                     hetlabel1, taulabel1)
+  meta1 <- updateobj(meta1,
+                     label1.common, label1.random, label1.predict,
+                     hetlabel1, taulabel1, label1.subgroup,
+                     text.common1, text.random1, text.predict1,
+                     text.w.common1, text.w.random1)
   ##
-  meta2 <- updateobj(meta2, text.common2, text.random2, text.predict2,
-                     text.w.common2, text.w.random2,
-                     hetlabel2, taulabel2)
+  hetlabel1 <- meta1$hetlabel
+  taulabel1 <- meta1$taulabel
+  ##
+  meta2 <- updateobj(meta2,
+                     label2.common, label2.random, label2.predict,
+                     hetlabel2, taulabel2, label2.subgroup,
+                     text.common2, text.random2, text.predict2,
+                     text.w.common2, text.w.random2)
+  ##
+  hetlabel2 <- meta2$hetlabel
+  taulabel2 <- meta2$taulabel
   
   
   ##
@@ -580,8 +652,8 @@ metamerge <- function(meta1, meta2,
   meta1$method.tau.ci <- replaceNULL(meta1$method.tau.ci, "")
   meta2$method.tau.ci <- replaceNULL(meta2$method.tau.ci, "")
   ##
-  meta1$method.random.ci <- replaceNULL(meta1$method.random.ci, "")
-  meta2$method.random.ci <- replaceNULL(meta2$method.random.ci, "")
+  #meta1$method.random.ci <- replaceNULL(meta1$method.random.ci, "")
+  #meta2$method.random.ci <- replaceNULL(meta2$method.random.ci, "")
   ##
   meta1$df.random <- replaceNULL(meta1$df.random, NA)
   meta2$df.random <- replaceNULL(meta2$df.random, NA)
@@ -639,11 +711,11 @@ metamerge <- function(meta1, meta2,
     res$text.w.common <- meta2$text.w.common
   }
   else if (!is.null(meta1$w.common) & !is.null(meta2$w.common) & keep.w) {
-    res$w.common <- cbind(meta1$w.common, meta2$w.common)
-    colnames(res$w.common) <- c(meta1$text.w.common, meta2$text.w.common)
-    rownames(res$w.common) <- meta1$studlab
-    ##
     res$text.w.common <- c(meta1$text.w.common, meta2$text.w.common)
+    ##
+    res$w.common <- cbind(meta1$w.common, meta2$w.common)
+    colnames(res$w.common) <- res$text.w.common
+    rownames(res$w.common) <- meta1$studlab
   }
   ##
   if (is.null(res$w.random) & !is.null(meta2$w.random)) {
@@ -651,11 +723,19 @@ metamerge <- function(meta1, meta2,
     res$text.w.random <- meta2$text.w.random
   }
   else if (!is.null(res$w.random) & !is.null(meta2$w.random) & keep.w) {
-    res$w.random <- cbind(meta1$w.random, meta2$w.random)
-    colnames(res$w.random) <- c(meta1$text.w.random, meta2$text.w.random)
-    rownames(res$w.random) <- meta1$studlab
-    ##
-    res$text.w.random <- c(meta1$text.w.random, meta2$text.w.random)
+    if (length(res$w.random) != length(meta2$w.random)) {
+      warning("Argument 'keep.w' set to FALSE as number of weights differs ",
+              "between meta-analyses.",
+              call. = FALSE)
+      keep.w <- FALSE
+    }
+    else {
+      res$text.w.random <- c(meta1$text.w.random, meta2$text.w.random)
+      ##
+      res$w.random <- cbind(meta1$w.random, meta2$w.random)
+      colnames(res$w.random) <- c(meta1$text.w.random, meta2$text.w.random)
+      rownames(res$w.random) <- meta1$studlab
+    }
   }
   ##
   res$hetlabel <- c(meta1$hetlabel, meta2$hetlabel)
@@ -669,24 +749,33 @@ metamerge <- function(meta1, meta2,
   ##
   ## Number of studies
   ##
-  res$k <-
-    expandmerge(meta1$k, meta2$k,
-                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
-  res$k.all <-
-    expandmerge(meta1$k.all, meta2$k.all,
-                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
-  res$k.MH <-
-    expandmerge(meta1$k.MH, meta2$k.MH,
-                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
-  res$k.study <-
-    expandmerge(meta1$k.study, meta2$k.study,
-                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
-  res$k.TE <-
-    expandmerge(meta1$k.TE, meta2$k.TE,
-                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
-  res$k0 <-
-    expandmerge(meta1$k0, meta2$k0,
-                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
+  if (!inherits(meta1, "metamerge") & (is.limit | is.copas | is.robu)) {
+    res$k <- meta1$k
+    res$k.all <- meta1$k.all
+    res$k.MH <- meta1$k.MH
+    res$k.study <- meta1$k.study
+    res$k.TE <- meta1$k.TE
+  }
+  else {
+    res$k <-
+      expandmerge(meta1$k, meta2$k,
+                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
+    res$k.all <-
+      expandmerge(meta1$k.all, meta2$k.all,
+                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
+    res$k.MH <-
+      expandmerge(meta1$k.MH, meta2$k.MH,
+                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
+    res$k.study <-
+      expandmerge(meta1$k.study, meta2$k.study,
+                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
+    res$k.TE <-
+      expandmerge(meta1$k.TE, meta2$k.TE,
+                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
+    res$k0 <-
+      expandmerge(meta1$k0, meta2$k0,
+                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
+  }
   ##
   ## Common effect model
   ##
@@ -705,7 +794,8 @@ metamerge <- function(meta1, meta2,
   res$seTE.random <- c(meta1$seTE.random, meta2$seTE.random)
   res$statistic.random <- c(meta1$statistic.random, meta2$statistic.random)
   res$pval.random <- c(meta1$pval.random, meta2$pval.random)
-  res$method.random.ci <- c(meta1$method.random.ci, meta2$method.random.ci)
+  res$method.random.ci <-
+    replaceNULL(c(meta1$method.random.ci, meta2$method.random.ci), "")
   res$df.random <- c(meta1$df.random, meta2$df.random)
   res$lower.random <- c(meta1$lower.random, meta2$lower.random)
   res$upper.random <- c(meta1$upper.random, meta2$upper.random)
@@ -723,45 +813,43 @@ metamerge <- function(meta1, meta2,
   ## Prediction interval
   ##
   res$method.predict <- c(meta1$method.predict, meta2$method.predict)
-  res$adhoc.hakn.pi <- c(meta1$adhoc.hakn.pi, meta2$adhoc.hakn.pi)
   res$seTE.predict <- c(meta1$seTE.predict, meta2$seTE.predict)
   res$df.predict <- c(meta1$df.predict, meta2$df.predict)
   res$lower.predict <- c(meta1$lower.predict, meta2$lower.predict)
   res$upper.predict <- c(meta1$upper.predict, meta2$upper.predict)
+  res$adhoc.hakn.pi <- c(meta1$adhoc.hakn.pi, meta2$adhoc.hakn.pi)
   res$seTE.hakn.pi <- c(meta1$seTE.hakn.pi, meta2$seTE.hakn.pi)
   res$seTE.hakn.adhoc.pi <-
     c(meta1$seTE.hakn.adhoc.pi, meta2$seTE.hakn.adhoc.pi)
   res$text.predict <- c(meta1$text.predict, meta2$text.predict)
   ##
+  res$prediction.subgroup <-
+    any(c(meta1$prediction.subgroup, meta2$prediction.subgroup))
+  ##
   ## Heterogeneity statistics
   ##
-  Qlabel1 <- if (missing.hetlabel1) names(meta1$Q) else hetlabel1
-  Qlabel2 <- if (missing.hetlabel2) names(meta2$Q) else hetlabel2
-  ##
   if (keep.Q) {
-    res$Q <- mergevars(meta1$Q, meta2$Q, Qlabel1, Qlabel2)
-    res$df.Q <- mergevars(meta1$df.Q, meta2$df.Q)
-    res$pval.Q <- mergevars(meta1$pval.Q, meta2$pval.Q)
+    res$Q <- c(meta1$Q, meta2$Q)
+    res$df.Q <- c(meta1$df.Q, meta2$df.Q)
+    res$pval.Q <- c(meta1$pval.Q, meta2$pval.Q)
   }
   ##
   if (keep.I2) {
-    res$I2 <- mergevars(meta1$I2, meta2$I2, Qlabel1, Qlabel2)
-    res$lower.I2 <- mergevars(meta1$lower.I2, meta2$upper.I2)
-    res$upper.I2 <- mergevars(meta1$lower.I2, meta2$upper.I2)
+    res$I2 <- c(meta1$I2, meta2$I2)
+    res$lower.I2 <- c(meta1$lower.I2, meta2$upper.I2)
+    res$upper.I2 <- c(meta1$upper.I2, meta2$upper.I2)
     ##
-    res$H <- mergevars(meta1$H, meta2$H, Qlabel1, Qlabel2)
-    res$lower.H <- mergevars(meta1$lower.H, meta2$upper.H)
-    res$upper.H <- mergevars(meta1$lower.H, meta2$upper.H)
+    res$H <- c(meta1$H, meta2$H)
+    res$lower.H <- c(meta1$lower.H, meta2$upper.H)
+    res$upper.H <- c(meta1$upper.H, meta2$upper.H)
     ##
-    res$Rb <- mergevars(meta1$Rb, meta2$Rb, Qlabel1, Qlabel2)
-    res$lower.Rb <- mergevars(meta1$lower.Rb, meta2$upper.Rb)
-    res$upper.Rb <- mergevars(meta1$lower.Rb, meta2$upper.Rb)
+    res$Rb <- c(meta1$Rb, meta2$Rb)
+    res$lower.Rb <- c(meta1$lower.Rb, meta2$upper.Rb)
+    res$upper.Rb <- c(meta1$upper.Rb, meta2$upper.Rb)
   }
   ##
   ## Trim-and-fill method
   ##
-  expandmerge(meta1$method.tau, meta2$method.tau,
-              nr1 = nran1, nr2 = nran2)
   res$left <-
     expandmerge(meta1$left, meta2$left,
                 nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
@@ -780,50 +868,52 @@ metamerge <- function(meta1, meta2,
   ##
   ## Subgroup analyses
   ##
-  res$TE.common.w <- mergevars(meta1$TE.common.w, meta2$TE.common.w)
-  res$seTE.common.w <- mergevars(meta1$seTE.common.w, meta2$seTE.common.w)
+  replaceNULL(label1.subgroup, "")
+  replaceNULL(label2.subgroup, "")
+  ##
+  res$TE.common.w <- c(meta1$TE.common.w, meta2$TE.common.w)
+  res$seTE.common.w <- c(meta1$seTE.common.w, meta2$seTE.common.w)
   res$statistic.common.w <-
-    mergevars(meta1$statistic.common.w, meta2$statistic.common.w)
-  res$pval.common.w <- mergevars(meta1$pval.common.w, meta2$pval.common.w)
-  res$lower.common.w <- mergevars(meta1$lower.common.w, meta2$lower.common.w)
-  res$upper.common.w <- mergevars(meta1$upper.common.w, meta2$upper.common.w)
+    c(meta1$statistic.common.w, meta2$statistic.common.w)
+  res$pval.common.w <- c(meta1$pval.common.w, meta2$pval.common.w)
+  res$lower.common.w <- c(meta1$lower.common.w, meta2$lower.common.w)
+  res$upper.common.w <- c(meta1$upper.common.w, meta2$upper.common.w)
   ##
-  res$w.common.w <- mergevars(meta1$w.common.w, meta2$w.common.w)
+  res$w.common.w <- c(meta1$w.common.w, meta2$w.common.w)
   ##
-  res$Q.w.common <- mergevars(meta1$Q.w.common, meta2$Q.w.common)
-  res$pval.Q.w.common <-
-    mergevars(meta1$pval.Q.w.common, meta2$pval.Q.w.common)
+  res$tau.w <- c(meta1$tau.w, meta2$tau.w)
   ##
-  res$Q.b.common <- mergevars(meta1$Q.b.common, meta2$Q.b.common)
-  res$df.Q.b.common <- mergevars(meta1$df.Q.b.common, meta2$df.Q.b.common)
-  res$pval.Q.b.common <-
-    mergevars(meta1$pval.Q.b.common, meta2$pval.Q.b.common)
+  res$Q.w.common <- c(meta1$Q.w.common, meta2$Q.w.common)
+  res$pval.Q.w.common <- c(meta1$pval.Q.w.common, meta2$pval.Q.w.common)
   ##
-  res$TE.random.w <- mergevars(meta1$TE.random.w, meta2$TE.random.w)
-  res$seTE.random.w <- mergevars(meta1$seTE.random.w, meta2$seTE.random.w)
+  res$Q.b.common <- c(meta1$Q.b.common, meta2$Q.b.common)
+  res$df.Q.b.common <- c(meta1$df.Q.b.common, meta2$df.Q.b.common)
+  res$pval.Q.b.common <- c(meta1$pval.Q.b.common, meta2$pval.Q.b.common)
+  ##
+  res$TE.random.w <- c(meta1$TE.random.w, meta2$TE.random.w)
+  res$seTE.random.w <- c(meta1$seTE.random.w, meta2$seTE.random.w)
   res$statistic.random.w <-
-    mergevars(meta1$statistic.random.w, meta2$statistic.random.w)
-  res$pval.random.w <- mergevars(meta1$pval.random.w, meta2$pval.random.w)
-  res$df.random.w <- list(meta1$df.random.w, meta2$df.random.w)
-  res$lower.random.w <- mergevars(meta1$lower.random.w, meta2$lower.random.w)
-  res$upper.random.w <- mergevars(meta1$upper.random.w, meta2$upper.random.w)
-  res$df.hakn.w <- list(meta1$df.hakn.w, meta2$df.hakn.w)
-  res$df.kero.w <- list(meta1$df.kero.w, meta2$df.kero.w)
+    c(meta1$statistic.random.w, meta2$statistic.random.w)
+  res$pval.random.w <- c(meta1$pval.random.w, meta2$pval.random.w)
+  res$df.random.w <- c(meta1$df.random.w, meta2$df.random.w)
+  res$lower.random.w <- c(meta1$lower.random.w, meta2$lower.random.w)
+  res$upper.random.w <- c(meta1$upper.random.w, meta2$upper.random.w)
+  res$df.hakn.w <- c(meta1$df.hakn.w, meta2$df.hakn.w)
+  res$df.kero.w <- c(meta1$df.kero.w, meta2$df.kero.w)
   ##
-  res$w.random.w <- mergevars(meta1$w.random.w, meta2$w.random.w)
+  res$w.random.w <- c(meta1$w.random.w, meta2$w.random.w)
   ##
-  res$Q.w.random <- mergevars(meta1$Q.w.random, meta2$Q.w.random)
-  res$pval.Q.w.random <-
-    mergevars(meta1$pval.Q.w.random, meta2$pval.Q.w.random)
+  res$Q.w.random <- c(meta1$Q.w.random, meta2$Q.w.random)
+  res$pval.Q.w.random <- c(meta1$pval.Q.w.random, meta2$pval.Q.w.random)
   ##
-  res$Q.b.random <- list(meta1$Q.b.random, meta2$Q.b.random)
-  res$df.Q.b.random <- list(meta1$df.Q.b.random, meta2$df.Q.b.random)
-  res$pval.Q.b.random <- mergevars(meta1$pval.Q.b.random, meta2$pval.Q.b.random)
+  res$Q.b.random <- c(meta1$Q.b.random, meta2$Q.b.random)
+  res$df.Q.b.random <- c(meta1$df.Q.b.random, meta2$df.Q.b.random)
+  res$pval.Q.b.random <- c(meta1$pval.Q.b.random, meta2$pval.Q.b.random)
   ##
-  res$seTE.predict.w <- mergevars(meta1$seTE.predict.w, meta2$seTE.predict.w)
-  res$df.predict.w <- list(meta1$df.predict.w, meta2$df.predict.w)
-  res$lower.predict.w <- mergevars(meta1$lower.predict.w, meta2$lower.predict.w)
-  res$upper.predict.w <- mergevars(meta1$upper.predict.w, meta2$upper.predict.w)
+  res$seTE.predict.w <- c(meta1$seTE.predict.w, meta2$seTE.predict.w)
+  res$df.predict.w <- c(meta1$df.predict.w, meta2$df.predict.w)
+  res$lower.predict.w <- c(meta1$lower.predict.w, meta2$lower.predict.w)
+  res$upper.predict.w <- c(meta1$upper.predict.w, meta2$upper.predict.w)
   
   
   ##
@@ -831,8 +921,10 @@ metamerge <- function(meta1, meta2,
   ## (7) More settings
   ##
   ##
-  taulabel1 <- if (missing.taulabel1) names(meta1$tau) else taulabel1
-  taulabel2 <- if (missing.taulabel2) names(meta2$tau) else taulabel2
+  if (is.null(taulabel1))
+    taulabel1 <- names(meta1$tau)
+  if (is.null(taulabel2))
+    taulabel2 <- names(meta2$tau)
   ##
   names(meta1$tau) <- names(meta1$tau2) <- taulabel1
   names(meta2$tau) <- names(meta2$tau2) <- taulabel2
@@ -859,15 +951,6 @@ metamerge <- function(meta1, meta2,
     res$Q.Cochrane <- meta2$Q.Cochrane
   }
   else if (random1 & random2) {
-    res$method.tau <-
-      expandmerge(meta1$method.tau, meta2$method.tau,
-                  nr1 = nran1, nr2 = nran2)
-    res$method.tau.ci <-
-      expandmerge(meta1$method.tau.ci, meta2$method.tau.ci,
-                  nr1 = nran1, nr2 = nran2)
-    ##
-    res$detail.tau <- c(meta1$detail.tau, meta2$detail.tau)
-    ##
     if (!inherits(meta1, "metamerge") && meta1$three.level) {
         meta1$tau2 <- sum(meta1$tau2)
         meta1$tau <- sqrt(meta1$tau2)
@@ -884,6 +967,13 @@ metamerge <- function(meta1, meta2,
         meta2$sign.lower.tau <- meta2$sign.upper.tau <- ""
     }
     ##
+    res$method.tau <-
+      expandmerge(meta1$method.tau, meta2$method.tau,
+                  nr1 = nran1, nr2 = nran2)
+    res$method.tau.ci <-
+      expandmerge(meta1$method.tau.ci, meta2$method.tau.ci,
+                  nr1 = nran1, nr2 = nran2)
+    ##
     res$tau <- expandmerge(meta1$tau, meta2$tau, nr1 = nran1, nr2 = nran2)
     res$lower.tau <-
       expandmerge(meta1$lower.tau, meta2$lower.tau, nr1 = nran1, nr2 = nran2)
@@ -899,6 +989,10 @@ metamerge <- function(meta1, meta2,
     res$se.tau <-
       expandmerge(meta1$se.tau, meta2$se.tau, nr1 = nran1, nr2 = nran2)
     ##
+    res$detail.tau <-
+      expandmerge(meta1$detail.tau, meta2$detail.tau,
+                  nr1 = nran1, nr2 = nran2)
+    ##
     res$tau.preset <-
       expandmerge(meta1$tau.preset, meta2$tau.preset, nr1 = nran1, nr2 = nran2)
     ##
@@ -908,9 +1002,9 @@ metamerge <- function(meta1, meta2,
   ##
   res$sm <- if (meta1$sm != meta2$sm) "" else meta1$sm
   ##
-  res$common <- common
-  res$random <- random
-  res$overall <- overall
+  res$common <- any(common)
+  res$random <- any(random)
+  res$overall <- any(overall)
   res$overall.hetstat <- overall.hetstat
   res$prediction <- prediction
   ##
@@ -918,9 +1012,16 @@ metamerge <- function(meta1, meta2,
   ##
   ## Three-level model
   ##
-  res$three.level <-
-    expandmerge(meta1$three.level, meta2$three.level,
-                ncom1, nran1, ncom2, nran2)
+  if (!inherits(meta1, "metamerge") & is.limit)
+    res$three.level <- unique(res$three.level)
+  else
+    res$three.level <-
+      expandmerge(meta1$three.level, meta2$three.level,
+                  nr1 = nran1, nr2 = nran2)
+  ##
+  res$rho <-
+    expandmerge(meta1$rho, meta2$rho, nr1 = nran1, nr2 = nran2)
+  ##
   if (!random1 & random2)
     res$cluster <- meta2$cluster
   ##
@@ -975,8 +1076,22 @@ metamerge <- function(meta1, meta2,
   ##
   ##
   res <- backward(res)
+  
+  
+  ##
+  ##
+  ## (9) Set class
+  ##
+  ##
   ##  
   class(res) <- c(class(res), "metamerge")
+  ##
+  if (inherits(meta2, "trimfill"))
+    class(res) <- c(class(res), "trimfill")
+  ##
+  res$call <- match.call()
+  ##
+  class(res) <- unique(class(res))
   
   
   res
