@@ -26,6 +26,7 @@
 #' Examples) or use one of the following general settings:
 #' \itemize{
 #' \item \code{settings.meta("RevMan5")}
+#' \item \code{settings.meta("BMJ")}
 #' \item \code{settings.meta("JAMA")}
 #' \item \code{settings.meta("IQWiG5")}
 #' \item \code{settings.meta("IQWiG6")}
@@ -38,7 +39,9 @@
 #' \url{https://training.cochrane.org/online-learning/core-software/revman})
 #' and specifies to use a RevMan 5 layout in forest plots.
 #'
-#' The second command can be used to generate forest plots following
+#' The second command can be used to generate forest plots in BMJ layout.
+#'
+#' The third command can be used to generate forest plots following
 #' instructions for authors of the \emph{Journal of the American
 #' Medical Association}
 #' (\url{https://jamanetwork.com/journals/jama/pages/instructions-for-authors/}).Study
@@ -60,14 +63,18 @@
 #' RevMan 5 settings, in detail:
 #' \tabular{lll}{
 #' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
-#' \code{method.random.ci} \tab "classic" \tab only available method in RevMan 5 \cr
+#' \code{method.random.ci} \tab "classic" \tab only available method
+#'   in RevMan 5 \cr
 #' \code{method.tau} \tab "DL" \tab only available method in RevMan 5
 #'   \cr
 #' \code{tau.common} \tab FALSE \tab common between-study variance in
 #'   subgroups \cr
 #' \code{MH.exact} \tab FALSE \tab exact Mantel-Haenszel method \cr
 #' \code{RR.Cochrane} \tab TRUE \tab calculation of risk ratios \cr
-#' \code{Q.Cochrane} \tab TRUE \tab calculation of heterogeneity statistic \cr
+#' \code{Q.Cochrane} \tab TRUE \tab calculation of heterogeneity
+#'   statistic \cr
+#' \code{exact.smd} \tab FALSE \tab exact formulae for Hedges' g and
+#'   Cohen's d \cr
 #' \code{layout} \tab "RevMan5" \tab layout for forest plots \cr
 #' \code{prediction} \tab FALSE \tab no prediction interval \cr
 #' \code{test.overall} \tab TRUE \tab print information on test of
@@ -76,17 +83,73 @@
 #'   subgroup differences \cr
 #' \code{test.effect.subgroup} \tab TRUE \tab print information on
 #'   test for effect in subgroups \cr
-#' \code{digits.I2} \tab 0 \tab number of digits for I-squared measure
+#' \code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
+#'   forest plots
+#'   \cr
+#' \code{forest.tau2} \tab TRUE \tab show between-study heterogeneity variance
+#'   in forest plots
+#'   \cr
+#' \code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity
+#'   standard deviation in forest plots
+#'   \cr
+#' \code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
+#'   forest plots
+#'   \cr
+#' \code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
+#'   in forest plots
+#'   \cr
+#' \code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
+#'   forest plots
 #'   \cr
 #' \code{digits.tau2} \tab 3 \tab number of digits for tau-squared \cr
 #' \code{digits.tau} \tab 4 \tab number of digits for square root of
 #'   tau-squared \cr
+#' \code{digits.I2} \tab 0 \tab number of digits for I-squared measure
+#'   \cr
 #' \code{CIbracket}, \tab "[" \tab \cr
 #' \code{CIseparator} \tab ", " \tab print confidence intervals as
 #'   "\code{[., .]}" \cr
 #' \code{header.line}, \tab TRUE \tab print header line
 #' }
 #'
+#' BMJ settings:
+#' \tabular{lll}{
+#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
+#' \code{layout} \tab "BMJ" \tab layout for forest plots \cr
+#' \code{test.overall} \tab TRUE \tab print information on test of
+#'   overall effect \cr
+#' \code{test.subgroup} \tab FALSE \tab print information on test for
+#'   subgroup differences \cr
+#' \code{test.effect.subgroup} \tab FALSE \tab print information on
+#'   test for effect in subgroups \cr
+#' \code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
+#'   forest plots
+#'   \cr
+#' \code{forest.tau2} \tab TRUE \tab show between-study heterogeneity variance
+#'   in forest plots
+#'   \cr
+#' \code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity
+#'   standard deviation in forest plots
+#'   \cr
+#' \code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
+#'   forest plots
+#'   \cr
+#' \code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
+#'   in forest plots
+#'   \cr
+#' \code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
+#'   forest plots
+#'   \cr
+#' \code{digits.I2} \tab 0 \tab number of digits for I-squared measure
+#'   \cr
+#' \code{digits.pval} \tab 2 \tab number of digits for p-values \cr
+#' \code{CIbracket}, \tab "(" \tab \cr
+#' \code{CIseparator} \tab " to " \tab print confidence intervals as
+#'   "\code{(. to .)}" \cr
+#' \code{hetlab}, \tab \tab "Test for heterogeneity: " \cr
+#' \code{header.line}, \tab TRUE \tab print header line
+#' }
+#'
 #' JAMA settings:
 #' \tabular{lll}{
 #' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
@@ -97,13 +160,31 @@
 #'   subgroup differences \cr
 #' \code{test.effect.subgroup} \tab FALSE \tab print information on
 #'   test for effect in subgroups \cr
+#' \code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
+#'   forest plots
+#'   \cr
+#' \code{forest.tau2} \tab FALSE \tab do not show between-study heterogeneity
+#'   variance in forest plots
+#'   \cr
+#' \code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity
+#'   standard deviation in forest plots
+#'   \cr
+#' \code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
+#'   forest plots
+#'   \cr
+#' \code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
+#'   in forest plots
+#'   \cr
+#' \code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
+#'   forest plots
+#'   \cr
 #' \code{digits.I2} \tab 0 \tab number of digits for I-squared measure
 #'   \cr
 #' \code{digits.pval} \tab 3 \tab number of digits for p-values \cr
 #' \code{CIbracket}, \tab "(" \tab \cr
 #' \code{CIseparator} \tab "-" \tab print confidence intervals as
 #'   "\code{(.-.)}" \cr
-#' \code{zero.pval}, \tab TRUE \tab print p-values with leading zero
+#' \code{zero.pval}, \tab FALSE \tab print p-values with leading zero
 #' \cr
 #' \code{JAMA.pval}, \tab TRUE \tab round p-values to three digits
 #'   (for 0.001 < p \eqn{\le} 0.01) or two digits (p > 0.01) \cr
@@ -258,7 +339,8 @@ settings.meta <- function(..., quietly = TRUE) {
   ##
   ## Set internal variables
   ##
-  settings <- c("RevMan5", "JAMA", "IQWiG5", "IQWiG6", "geneexpr", "meta4")
+  settings <- c("BMJ", "JAMA", "RevMan5", 
+                "IQWiG5", "IQWiG6", "geneexpr", "meta4")
   layouts <- c(settings[1:2], "meta")
   ##
   print.settings <- FALSE
@@ -424,6 +506,8 @@ settings.meta <- function(..., quietly = TRUE) {
     setOption("test.subgroup", TRUE)
     setOption("prediction.subgroup", FALSE)
     setOption("method.bias", "Egger")
+    setOption("tool.rob", NULL)
+    setOption("overall.hetstat", NULL)
     setOption("text.common", "Common effect model")
     setOption("text.random", "Random effects model")
     setOption("text.predict", "Prediction interval")
@@ -445,6 +529,8 @@ settings.meta <- function(..., quietly = TRUE) {
     setOption("transf", TRUE)
     setOption("backtransf", TRUE)
     setOption("digits", 4)
+    setOption("digits.mean", 2)
+    setOption("digits.sd", 4)
     setOption("digits.se", 4)
     setOption("digits.stat", 2)
     setOption("digits.zval", 2)
@@ -462,14 +548,24 @@ settings.meta <- function(..., quietly = TRUE) {
     setOption("zero.pval", TRUE)
     setOption("JAMA.pval", FALSE)
     setOption("digits.df", 4)
+    ##
+    setOption("details", TRUE)
+    ##
+    setOption("print.tau2", TRUE)
+    setOption("print.tau2.ci", TRUE)
+    setOption("print.tau", TRUE)
+    setOption("print.tau.ci", TRUE)
     setOption("print.I2", TRUE)
     setOption("print.H", TRUE)
     setOption("print.Rb", FALSE)
+    ##
     setOption("text.tau2", "tau^2")
     setOption("text.tau", "tau")
     setOption("text.I2", "I^2")
     setOption("text.Rb", "Rb")
     ##
+    setOption("print.Q", TRUE)
+    ##
     ## R function metabin
     ##
     setOption("smbin", "RR")
@@ -520,6 +616,7 @@ settings.meta <- function(..., quietly = TRUE) {
     ## R function forest.meta
     ##
     setOption("layout", "meta")
+    setOption("forest.details", FALSE)
     setOption("test.overall", FALSE)
     setOption("test.effect.subgroup", FALSE)
     setOption("digits.forest", 2)
@@ -541,20 +638,36 @@ settings.meta <- function(..., quietly = TRUE) {
     setOption("lty.equi", 1)
     setOption("col.equi", "blue")
     setOption("fill.equi", "transparent")
+    setOption("fill", "transparent")
+    ##
+    setOption("leftcols", NULL)
+    setOption("rightcols", NULL)
+    setOption("leftlabs", NULL)
+    setOption("rightlabs", NULL)
+    ##
+    setOption("label.e.attach", NULL)
+    setOption("label.c.attach", NULL)
     ##
     setOption("bottom.lr", TRUE)
     ##
     setOption("lab.NA", ".")
     setOption("lab.NA.effect", NULL)
-    setOption("lab.NA.weight", "--")
+    setOption("lab.NA.weight", ".")
     ##
     setOption("lwd", 1)
+    setOption("lwd.square", 1)
+    setOption("lwd.diamond", 1)
+    ##
+    setOption("arrow.type", "open")
+    setOption("arrow.length", 0.05)
     ##
     setOption("type.study", "square")
     setOption("type.common", "diamond")
     ##
     setOption("col.study", "black")
     setOption("col.square", "gray")
+    setOption("col.square.lines", "gray")
+    setOption("col.circle", "royalblue")
     setOption("col.inside", "white")
     setOption("col.diamond", "gray")
     setOption("col.diamond.lines", "black")
@@ -564,6 +677,9 @@ settings.meta <- function(..., quietly = TRUE) {
     setOption("col.label.right", "black")
     setOption("col.label.left", "black")
     ##
+    setOption("col.lines", "black")
+    setOption("col.label", "black")
+    ##
     setOption("hetlab", "Heterogeneity: ")
     setOption("resid.hetstat", NULL)
     setOption("resid.hetlab", "Residual heterogeneity: ")
@@ -621,6 +737,9 @@ settings.meta <- function(..., quietly = TRUE) {
     setOption("ff.lr", "plain")
     ##
     setOption("colgap", "2mm")
+    setOption("colgap.forest", "2mm")
+    ##
+    setOption("width", NULL)
     ##
     setOption("calcwidth.predict", FALSE)
     setOption("calcwidth.hetstat", FALSE)
@@ -643,69 +762,179 @@ settings.meta <- function(..., quietly = TRUE) {
   ## Specific settings
   ##
   if (specific.settings) {
-    ##
-    ## Remember:
-    ## settings <- c("RevMan5", "JAMA", "IQWiG5", "IQWiG6",
-    ##               "geneexpr", "meta4")
-    ##
-    if (setting == "RevMan5") {
+    #
+    if (setting == "BMJ") {
+      specificSettings(
+        args = c("layout", "test.overall",
+                 "test.subgroup", "test.effect.subgroup",
+                 #
+                 "forest.I2", "forest.tau2", "forest.tau",
+                 "forest.Q", "forest.pval.Q", "forest.Rb",
+                 #
+                 "digits.I2", "digits.tau2", "digits.tau", "digits.pval",
+                 "digits.forest", "digits.mean", "digits.sd",
+                 #
+                 "CIbracket", "CIseparator",
+                 #
+                 "colgap.forest",
+                 #
+                 "col.common", "col.random", "col.subgroup",
+                 "col.study", "col.square", "col.square.lines",
+                 "col.diamond", "col.diamond.lines",
+                 #
+                 "col.lines",
+                 #
+                 "lwd.square", "lwd.diamond",
+                 #
+                 "arrow.type",
+                 #
+                 "ff.lr",
+                 "zero.pval", "JAMA.pval",
+                 #
+                 "hetlab", "header.line"),
+        new = list("BMJ",
+                   replaceNULL(args[["test.overall"]], TRUE),
+                   replaceNULL(args[["test.subgroup"]], TRUE),
+                   replaceNULL(args[["test.effect.subgroup"]], TRUE),
+                   #
+                   replaceNULL(args[["forest.I2"]], TRUE),
+                   replaceNULL(args[["forest.tau2"]], TRUE),
+                   replaceNULL(args[["forest.tau"]], FALSE),
+                   replaceNULL(args[["forest.Q"]], TRUE),
+                   replaceNULL(args[["forest.pval.Q"]], TRUE),
+                   replaceNULL(args[["forest.Rb"]], FALSE),
+                   #
+                   replaceNULL(args[["digits.I2"]], 0),
+                   replaceNULL(args[["digits.tau2"]], 2),
+                   replaceNULL(args[["digits.tau"]], 2),
+                   replaceNULL(args[["digits.pval"]], 2),
+                   replaceNULL(args[["digits.forest"]], 2),
+                   replaceNULL(args[["digits.mean"]], 1),
+                   replaceNULL(args[["digits.sd"]], 1),
+                   #
+                   replaceNULL(args[["CIbracket"]], "("),
+                   replaceNULL(args[["CIseparator"]], " to "),
+                   #
+                   replaceNULL(args[["colgap.forest"]], "5mm"),
+                   #
+                   replaceNULL(args[["col.common"]], "#6b58a6"),
+                   replaceNULL(args[["col.random"]], "#6b58a6"),
+                   replaceNULL(args[["col.subgroup"]], "black"),
+                   replaceNULL(args[["col.study"]], "#6b58a6"),
+                   replaceNULL(args[["col.square"]], "#6b58a6"),
+                   replaceNULL(args[["col.square.lines"]], "white"),
+                   replaceNULL(args[["col.diamond"]], "#6b58a6"),
+                   replaceNULL(args[["col.diamond.lines"]], "white"),
+                   #
+                   replaceNULL(args[["col.lines"]], "#a7a9ac"),
+                   #
+                   replaceNULL(args[["lwd.square"]], 0.5),
+                   replaceNULL(args[["lwd.diamond"]], 0.5),
+                   #
+                   replaceNULL(args[["arrow.type"]], "closed"),
+                   #
+                   replaceNULL(args[["ff.lr"]], "bold"),
+                   replaceNULL(args[["zero.pval"]], TRUE),
+                   replaceNULL(args[["JAMA.pval"]], TRUE),
+                   #
+                   replaceNULL(args[["hetlab"]], "Test for heterogeneity: "),
+                   replaceNULL(args[["header.line"]], TRUE)),
+        setting = "BMJ settings",
+        quietly = quietly)
+    }
+    #
+    else if (setting == "JAMA") {
+      specificSettings(
+        args = c("layout", "test.overall",
+                 "test.subgroup", "test.effect.subgroup",
+                 #
+                 "forest.I2", "forest.tau2", "forest.tau",
+                 "forest.Q", "forest.pval.Q", "forest.Rb",
+                 #
+                 "digits.I2", "digits.pval",
+                 "CIbracket", "CIseparator",
+                 "zero.pval", "JAMA.pval",
+                 "hetlab", "header.line"),
+        new = list("JAMA",
+                   replaceNULL(args[["test.overall"]], TRUE),
+                   replaceNULL(args[["test.subgroup"]], FALSE),
+                   replaceNULL(args[["test.effect.subgroup"]], FALSE),
+                   #
+                   replaceNULL(args[["forest.tau2"]], FALSE),
+                   replaceNULL(args[["forest.tau"]], FALSE),
+                   replaceNULL(args[["forest.I2"]], TRUE),
+                   replaceNULL(args[["forest.Q"]], TRUE),
+                   replaceNULL(args[["forest.pval.Q"]], TRUE),
+                   replaceNULL(args[["forest.Rb"]], FALSE),
+                   #
+                   replaceNULL(args[["digits.I2"]], 0),
+                   replaceNULL(args[["digits.pval"]], 3),
+                   replaceNULL(args[["CIbracket"]], "("),
+                   replaceNULL(args[["CIseparator"]], "-"),
+                   replaceNULL(args[["zero.pval"]], FALSE),
+                   replaceNULL(args[["JAMA.pval"]], TRUE),
+                   replaceNULL(args[["hetlab"]], "Heterogeneity: "),
+                   replaceNULL(args[["header.line"]], TRUE)),
+        setting = "JAMA settings",
+        quietly = quietly)
+    }
+    #
+    else if (setting == "RevMan5") {
       specificSettings(
         args = c("method.random.ci", "method.tau", "tau.common",
                  "MH.exact", "RR.Cochrane", "Q.Cochrane",
+                 "exact.smd",
                  "layout", "prediction", "test.overall",
                  "test.subgroup", "test.effect.subgroup",
-                 "digits.I2", "digits.tau2", "digits.tau",
+                 "col.subgroup",
+                 #
+                 "forest.I2", "forest.tau2", "forest.tau",
+                 "forest.Q", "forest.pval.Q", "forest.Rb",
+                 #
+                 "digits", "digits.I2", "digits.tau2", "digits.tau",
+                 #
                  "CIbracket", "CIseparator",
                  "zero.pval", "JAMA.pval",
-                 "text.common", "text.w.common", "header.line"),
+                 "text.common", "text.w.common",
+                 "hetlab", "header.line"),
         new = list(replaceNULL(args[["method.random.ci"]], "classic"),
                    replaceNULL(args[["method.tau"]], "DL"),
                    replaceNULL(args[["tau.common"]], FALSE),
                    replaceNULL(args[["MH.exact"]], FALSE),
                    replaceNULL(args[["RR.Cochrane"]], TRUE),
                    replaceNULL(args[["Q.Cochrane"]], TRUE),
-                   "RevMan5",
+                   replaceNULL(args[["exact.smd"]], FALSE), "RevMan5",
                    replaceNULL(args[["prediction"]], FALSE),
                    replaceNULL(args[["test.overall"]], TRUE),
                    replaceNULL(args[["test.subgroup"]], TRUE),
                    replaceNULL(args[["test.effect.subgroup"]], TRUE),
+                   replaceNULL(args[["col.subgroup"]], "black"),
+                   #
+                   replaceNULL(args[["forest.I2"]], TRUE),
+                   replaceNULL(args[["forest.tau2"]], TRUE),
+                   replaceNULL(args[["forest.tau"]], FALSE),
+                   replaceNULL(args[["forest.Q"]], TRUE),
+                   replaceNULL(args[["forest.pval.Q"]], TRUE),
+                   replaceNULL(args[["forest.Rb"]], FALSE),
+                   #
+                   replaceNULL(args[["digits"]], 2),
                    replaceNULL(args[["digits.I2"]], 0),
                    replaceNULL(args[["digits.tau2"]], 3),
                    replaceNULL(args[["digits.tau"]], 4),
+                   #
                    replaceNULL(args[["CIbracket"]], "["),
                    replaceNULL(args[["CIseparator"]], ", "),
-                   replaceNULL(args[["zero.pval"]], FALSE),
+                   replaceNULL(args[["zero.pval"]], TRUE),
                    replaceNULL(args[["JAMA.pval"]], FALSE),
                    replaceNULL(args[["text.common"]], "Fixed effect model"),
                    replaceNULL(args[["text.w.common"]], "fixed"),
+                   replaceNULL(args[["hetlab"]], "Heterogeneity: "),
                    replaceNULL(args[["header.line"]], TRUE)
                    ),
         setting = "RevMan 5 settings",
         quietly = quietly)
     }
     ##
-    else if (setting == "JAMA") {
-      specificSettings(args = c("layout", "test.overall",
-                                "test.subgroup", "test.effect.subgroup",
-                                "digits.I2", "digits.pval",
-                                "CIbracket", "CIseparator",
-                                "zero.pval", "JAMA.pval",
-                                "header.line"),
-                       new = list("JAMA",
-                                  replaceNULL(args[["test.overall"]], TRUE),
-                                  replaceNULL(args[["test.subgroup"]], FALSE),
-                                  replaceNULL(args[["test.effect.subgroup"]], FALSE),
-                                  replaceNULL(args[["digits.I2"]], 0),
-                                  replaceNULL(args[["digits.pval"]], 3),
-                                  replaceNULL(args[["CIbracket"]], "("),
-                                  replaceNULL(args[["CIseparator"]], "-"),
-                                  replaceNULL(args[["zero.pval"]], FALSE),
-                                  replaceNULL(args[["JAMA.pval"]], TRUE),
-                                  replaceNULL(args[["header.line"]], TRUE)),
-                       setting = "JAMA settings",
-                       quietly = quietly)
-    }
-    ##
     else if (setting == "IQWiG5") {
       specificSettings(args = c("method.random.ci", "prediction"),
                        new = list(replaceNULL(args[["method.random.ci"]],
@@ -781,6 +1010,8 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("test.subgroup      ")
     catarg("prediction.subgroup")
     catarg("method.bias        ")
+    catarg("tool.rob           ")
+    catarg("overall.hetstat    ")
     catarg("text.common        ")
     catarg("text.random        ")
     catarg("text.predict       ")
@@ -799,6 +1030,8 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("warn.deprecated    ")
     catarg("backtransf         ")
     catarg("digits             ")
+    catarg("digits.mean        ")
+    catarg("digits.sd          ")
     catarg("digits.se          ")
     catarg("digits.stat        ")
     catarg("digits.Q           ")
@@ -815,6 +1048,11 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("zero.pval          ")
     catarg("JAMA.pval          ")
     catarg("digits.df          ")
+    catarg("details            ")
+    catarg("print.tau2         ")
+    catarg("print.tau2.ci      ")
+    catarg("print.tau          ")
+    catarg("print.tau.ci       ")
     catarg("print.I2           ")
     catarg("print.H            ")
     catarg("print.Rb           ")
@@ -822,6 +1060,7 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("text.tau           ")
     catarg("text.I2            ")
     catarg("text.Rb            ")
+    catarg("print.Q            ")
     ##
     cat(paste("\n* Default summary measure (argument 'sm' in",
                "corresponding function) *\n"))
@@ -880,6 +1119,7 @@ settings.meta <- function(..., quietly = TRUE) {
     ##
     cat("\n* Settings for forest.meta() *\n")
     catarg("layout                 ")
+    catarg("forest.details         ")
     catarg("test.overall           ")
     catarg("test.effect.subgroup   ")
     catarg("digits.forest          ",
@@ -904,6 +1144,16 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("col.equi               ")
     catarg("fill.equi              ")
     ##
+    catarg("fill                   ")
+    ##
+    catarg("leftcols               ")
+    catarg("rightcols              ")
+    catarg("leftlabs               ")
+    catarg("rightlabs              ")
+    ##
+    catarg("label.e.attach         ")
+    catarg("label.c.attach         ")
+    ##
     catarg("bottom.lr              ")
     ##
     catarg("lab.NA                 ")
@@ -911,12 +1161,19 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("lab.NA.weight          ")
     ##
     catarg("lwd                    ")
+    catarg("lwd.square             ")
+    catarg("lwd.diamond            ")
+    ##
+    catarg("arrow.type             ")
+    catarg("arrow.length           ")
     ##
     catarg("type.study             ")
     catarg("type.common            ")
     ##
     catarg("col.study              ")
     catarg("col.square             ")
+    catarg("col.square.lines       ")
+    catarg("col.circle             ")
     catarg("col.inside             ")
     catarg("col.diamond            ")
     catarg("col.diamond.lines      ")
@@ -926,6 +1183,9 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("col.label.right        ")
     catarg("col.label.left         ")
     ##
+    catarg("col.lines              ")
+    catarg("col.label              ")
+    ##
     catarg("hetlab                 ")
     catarg("resid.hetstat          ")
     catarg("resid.hetlab           ")
@@ -983,6 +1243,9 @@ settings.meta <- function(..., quietly = TRUE) {
     catarg("ff.lr                  ")
     ##
     catarg("colgap                 ")
+    catarg("colgap.forest          ")
+    ##
+    catarg("width                  ")
     ##
     catarg("calcwidth.predict      ")
     catarg("calcwidth.hetstat      ")
@@ -1048,6 +1311,8 @@ settings.meta <- function(..., quietly = TRUE) {
     setlevel("level.predict", args)
     setcharacter("method.predict", args, gs("meth4pi"))
     setcharacter("method.bias", args, gs("meth4bias"))
+    setcharacter("tool.rob", args, gs("tool4rob"), NULL.ok = TRUE)
+    setlogical("overall.hetstat", args, NULL.ok = TRUE)
     setcharacter("text.common", args)
     setcharacter("text.random", args)
     setcharacter("text.predict", args)
@@ -1075,6 +1340,8 @@ settings.meta <- function(..., quietly = TRUE) {
     setlogical("warn.deprecated", args)
     setlogical("backtransf", args)
     setnumeric("digits", args)
+    setnumeric("digits.mean", args)
+    setnumeric("digits.sd", args)
     setnumeric("digits.se", args)
     ##
     na <- is.na(setnumeric("digits.stat", args))
@@ -1096,14 +1363,23 @@ settings.meta <- function(..., quietly = TRUE) {
     setlogical("zero.pval", args)
     setlogical("JAMA.pval", args)
     setnumeric("digits.df", args)
+    ##
+    setlogical("details", args)
+    setlogical("print.tau2", args)
+    setlogical("print.tau2.ci", args)
+    setlogical("print.tau", args)
+    setlogical("print.tau.ci", args)
     setlogical("print.I2", args)
     setlogical("print.H", args)
     setlogical("print.Rb", args)
+    ##
     setcharacter("text.tau2", args)
     setcharacter("text.tau", args)
     setcharacter("text.I2", args)
     setcharacter("text.Rb", args)
     ##
+    setlogical("print.Q", args)
+    ##
     ## R function metabin
     ##
     setcharacter("smbin", args, gs("sm4bin"))
@@ -1171,6 +1447,7 @@ settings.meta <- function(..., quietly = TRUE) {
     ## R function forest.meta
     ##
     setcharacter("layout", args, layouts)
+    setlogical("forest.details", args)
     setlogical("test.overall", args)
     setlogical("test.subgroup", args)
     setlogical("prediction.subgroup", args)
@@ -1195,6 +1472,18 @@ settings.meta <- function(..., quietly = TRUE) {
     setcolor("col.equi", args)
     setcolor("fill.equi", args)
     ##
+    setcolor("fill", args)
+    ##
+    setcharacter("leftcols", args, length = 0,
+                 NULL.ok = TRUE, logical.ok = TRUE)
+    setcharacter("rightcols", args, length = 0,
+                 NULL.ok = TRUE, logical.ok = TRUE)
+    setcharacter("leftlabs", args, length = 0, NULL.ok = TRUE)
+    setcharacter("rightlabs", args, length = 0, NULL.ok = TRUE)
+    ##
+    setcharacter("label.e.attach", args, NULL.ok = TRUE)
+    setcharacter("label.c.attach", args, NULL.ok = TRUE)
+    ##
     setlogical("bottom.lr", args)
     ##
     setcharacter("lab.NA", args)
@@ -1202,12 +1491,19 @@ settings.meta <- function(..., quietly = TRUE) {
     setcharacter("lab.NA.weight", args)
     ##
     setnumeric("lwd", args)
+    setnumeric("lwd.square", args)
+    setnumeric("lwd.diamond", args)
+    ##
+    setcharacter("arrow.type", args)
+    setnumeric("arrow.length", args)
     ##
     setcharacter("type.study", args, c("square", "diamond", "predict"))
     setcharacter("type.common", args, c("square", "diamond", "predict"))
     ##
     setcolor("col.study", args)
     setcolor("col.square", args)
+    setcolor("col.square.lines", args)
+    setcolor("col.circle", args)
     setcolor("col.inside", args)
     setcolor("col.diamond", args)
     setcolor("col.diamond.lines", args)
@@ -1217,6 +1513,9 @@ settings.meta <- function(..., quietly = TRUE) {
     setcolor("col.label.right", args)
     setcolor("col.label.left", args)
     ##
+    setcolor("col.lines", args)
+    setcolor("col.label", args)
+    ##
     setcharacter("hetlab", args)
     setlogical("resid.hetstat", args, TRUE)
     setcharacter("resid.hetlab", args)
@@ -1276,6 +1575,9 @@ settings.meta <- function(..., quietly = TRUE) {
     setcharacter("ff.lr", args)
     ##
     setcharacter("colgap", args)
+    setcharacter("colgap.forest", args)
+    ##
+    setnumeric("width", args, TRUE)
     ##
     setlogical("calcwidth.predict", args)
     setlogical("calcwidth.hetstat", args)
